/* $Id: packet.c,v 1.4 2001/02/21 23:43:16 pawelk Exp $ */

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <netinet/in.h>
#include "net.h"
#include "rpmpldi.h"
#include "md5.h"

#define u32 unsigned int

/*
 * versions of read and write that block
 * until all data is written/read
 */
static int xread(int fd, void *buf, int size)
{
	int n;
	while (size) {
		n = read(fd, buf, size);
		if (n <= 0)
			return -1;
		size -= n;
		((char *) buf) += n;
	}
	return 0;
}

static int xwrite(int fd, const void *buf, int size)
{
	int n;
	while (size) {
		n = write(fd, buf, size);
		if (n == -1)
			return -1;
		size -= n;
		((const char *) buf) += n;
	}
	return 0;
}

/*
 * add string entry to hash
 */
void pkt_add_string(struct phash *h, const char *name, const char *data)
{
	struct pkt_entry *e;

	e = ymalloc(sizeof(struct pkt_entry) + 
	    strlen(name) + 1 + strlen(data) + 1);
	e->name = (char*)(e + 1);
	e->string = e->name + strlen(name) + 1;
	strcpy(e->name, name);
	strcpy(e->string, data);
	e->number = atoi(e->string);
	ph_add(h, e);
}

/*
 * add integer entry to hash
 */
void pkt_add_int(struct phash *h, const char *name, int data)
{
	char buf[30];

	snprintf(buf, sizeof(buf) - 1, "%d", data);
	pkt_add_string(h, name, buf);
}

/*
 * send packet, free hash.
 */
int pkt_send(int fd, struct phash *h)
{
	struct dbuf *db;
	int i;
	u32 size;
	void *buf;
	struct pkt_entry *e;
	
	db = db_new();

	for (i = 0; i <= (int)h->mask; i++)
		for (e = (struct pkt_entry*)h->tab[i]; e; e = e->next) {
			db_add(db, e->name, strlen(e->name) + 1);
			db_add(db, e->string, strlen(e->string) + 1);
		}
	db_addc(db, 0);
	db_addc(db, 0);
	buf = db_finish(db, &i);
	size = htonl(i);
	ph_std_kill(h);

	if (xwrite(fd, &size, 4) || xwrite(fd, buf, i))
	    	return -1;

	return 0;
}

/*
 * recv packet. hash needs to get freed by pkt_free
 */
struct phash *pkt_recv(int fd)
{
	struct phash *pkt;
	char *buf, *e;
	u32 size;

	if (xread(fd, &size, 4))
		return NULL;
		
	size = ntohl(size);
	
	/* sth went wrong? */
	if (size > 0xffffff)
		return NULL;

	buf = ymalloc(size);
	if (xread(fd, buf, size)) {
		yfree(buf);
		return NULL;
	}

	pkt = ph_new();

	/* FIXME: this can be crashed by bad data. */
	while (*buf) {
		e = strchr(buf, 0) + 1;
		pkt_add_string(pkt, buf, e);
		buf = strchr(e, 0) + 1;
	}

	return pkt;
}

void pkt_free(struct phash *pkt)
{
	ph_std_kill(pkt);
}

/*
 * count md5 hash of nul terminated string.
 * put result in usual ascii form in junk, which has to be at least 
 * 33 bytes long.
 */
void pkt_md5(const char *string, char *junk)
{
	struct MD5Context ctx;
	unsigned char res[16];
	static char *hex = "0123456789abcdef";
	int i;

	MD5Init(&ctx);
	MD5Update(&ctx, string, strlen(string));
	MD5Final(res, &ctx);
	
	junk[32] = 0;
	for (i = 0; i < 16; i++) {
		junk[i * 2 +1] = hex[res[i] & 0xf];
		junk[i * 2] = hex[(res[i] >> 4) & 0xf];
	}
}

const char *pkt_get_string(struct phash *h, const char *name, const char *def)
{
	struct pkt_entry *e;

	if (h == 0)
		return def;
		
	e = ph_find(h, name);
	return e ? e->string : def;
}

int pkt_get_int(struct phash *h, const char *name, int def)
{
	struct pkt_entry *e;

	if (h == 0)
		return def;
		
	e = ph_find(h, name);
	return e ? e->number : def;
}

void *ymalloc(int s)
{
	void *r;

	r = malloc(s);
	
	if (r == 0) {
		fprintf(stderr, _("Out of memory\n"));
		exit(1);
	}
	
	return r;
}

void yfree(void *p)
{
	free(p);
}

char *ystrdup(const char *s)
{
	char *r;

	r = strdup(s);
	if (r == 0) {
		fprintf(stderr, _("Out of memory (strdup)\n"));
		exit(1);
	}

	return r;
}
